@@ -28,7 +28,7 @@ Approach:
 """
 
 from urllib.error import (URLError, HTTPError)
-from urllib.parse import urlparse
+from urllib.parse import (quote, urlparse)
 from urllib.request import urlopen
 import codecs
 import hashlib
@@ -298,10 +298,16 @@ class SourcePackage(object):
         return os.path.join(mirror, 'pool', self.component, group,
                             self.source, filename)
 
-    def _lp_url(self, filename):
-        "Build a source package URL on Launchpad"
-        return os.path.join('https://launchpad.net', self.distribution,
-                            '+archive', 'primary', '+files', filename)
+    def _lp_url(self, filename, source=False):
+        "Build an archive file URL on Launchpad"
+        if source:
+            suffix = '+sourcefiles/%s/%s/%s' % (
+                quote(self.source), quote(self.version.full_version),
+                quote(filename))
+        else:
+            suffix = '+files/%s' % quote(filename)
+        return 'https://launchpad.net/%s/+archive/primary/%s' % (
+            quote(self.distribution), suffix)
 
     def _source_urls(self, name):
         "Generator of sources for name"
@@ -312,7 +318,7 @@ class SourcePackage(object):
         for mirror in self.masters:
             if mirror not in self.mirrors:
                 yield self._mirror_url(mirror, name)
-        yield self._lp_url(name)
+        yield self._lp_url(name, source=True)
 
     def _binary_urls(self, name, default_urls):
         "Generator of URLs for name"
@@ -339,7 +345,7 @@ class SourcePackage(object):
                 parsed = urlparse(self._dsc_source)
             url = self._dsc_source
         else:
-            url = self._lp_url(self.dsc_name)
+            url = self._lp_url(self.dsc_name, source=True)
         self._download_dsc(url)
 
         self._check_dsc()
@@ -465,8 +471,11 @@ class SourcePackage(object):
 
         if urlparse(url).scheme in ["", "file"]:
             frompath = os.path.abspath(urlparse(url).path)
-            Logger.info("Copying %s from %s" % (filename, frompath))
-            shutil.copyfile(frompath, pathname)
+            if frompath == pathname:
+                Logger.info("Using %s" % pathname)
+            else:
+                Logger.info("Copying %s from %s" % (filename, frompath))
+                shutil.copyfile(frompath, pathname)
         else:
             try:
                 with closing(urlopen(url)) as f:
@@ -510,22 +519,23 @@ class SourcePackage(object):
             else:
                 raise DownloadError('File %s could not be found' % name)
 
-    def pull_binaries(self, arch, name=None, ext=None):
+    def pull_binaries(self, arch=None, name=None, ext=None):
         """Pull binary debs into workdir.
         If name is specified, only binary packages matching the regex are included.
 
         If ext is specified, only binary packages with that ext are included; for
         example to only download dbgsym ddebs, specify ext='ddeb'.
 
-        Must specify arch, or use 'all' to pull all archs.
+        If arch is not specified or is 'all', pull all archs.
+
         Returns the number of files downloaded.
         """
         total = 0
 
         Logger.debug("pull_binaries(arch=%s, name=%s, ext=%s)" % (arch, name, ext))
 
-        if not arch:
-            raise RuntimeError("Must specify arch")
+        if arch == 'all':
+            arch = None
 
         for (fname, furls, fsize) in self._binary_files_info(arch, name, ext):
             found = False
@@ -599,7 +609,7 @@ class DebianSPPH(SourcePackagePublishingHistory):
     """
     resource_type = 'source_package_publishing_history'
 
-    def getBinaries(self, arch, name=None, ext=None):
+    def getBinaries(self, arch=None, name=None, ext=None):
         Logger.info('Using Snapshot to find binary packages')
         srcpkg = Snapshot.getSourcePackage(self.getPackageName(),
                                            version=self.getVersion())
@@ -755,11 +765,17 @@ class PersonalPackageArchiveSourcePackage(UbuntuSourcePackage):
         self._team = None
         self._ppa = None
 
-    def _lp_url(self, filename):
-        "Build a source package URL on Launchpad"
-        return os.path.join('https://launchpad.net', '~' + self._ppateam,
-                            '+archive', self.distribution, self._ppaname,
-                            '+files', filename)
+    def _lp_url(self, filename, source=False):
+        "Build an archive file URL on Launchpad"
+        if source:
+            suffix = '+sourcefiles/%s/%s/%s' % (
+                quote(self.source), quote(self.version.full_version),
+                quote(filename))
+        else:
+            suffix = '+files/%s' % quote(filename)
+        return 'https://launchpad.net/~%s/+archive/%s/%s/%s' % (
+            quote(self._ppateam), quote(self.distribution),
+            quote(self._ppaname), suffix)
 
 
 class UbuntuCloudArchiveSourcePackage(PersonalPackageArchiveSourcePackage):
@@ -1213,7 +1229,7 @@ class SnapshotSPPH(object):
             new_entries.append(str(block))
         return ''.join(new_entries)
 
-    def getBinaries(self, arch, name=None, ext=None):
+    def getBinaries(self, arch=None, name=None, ext=None):
         return [b.getBPPH()
                 for b in self._pkg.getBinaryFiles(arch=arch, name=name, ext=ext)]
 
