@@ -699,7 +699,6 @@ class SourcePackagePublishingHistory(BaseWrapper):
         self._archive = None
         self._changelog = None
         self._binaries = {}
-        self._have_all_binaries = False
         self._distro_series = None
         # Don't share _builds between different
         # SourcePackagePublishingHistory objects
@@ -795,39 +794,29 @@ class SourcePackagePublishingHistory(BaseWrapper):
             new_entries.append(str(block))
         return ''.join(new_entries)
 
-    def getBinaries(self, arch, name=None, ext=None):
+    def getBinaries(self, arch=None, name=None, ext=None):
         '''
         Returns the resulting BinaryPackagePublishingHistorys.
-        Must specify arch, or use 'all' to get all archs.
+        If arch is specified, it returns binaries for only that arch,
+        plus any binaries with arch 'all'.  If arch is not specified, or
+        if arch is specified as 'all', all archs are returned.
 
         If name is specified, only returns BPPH matching that (regex) name.
 
         If ext is specified, only returns BPPH matching that (regex) ext.
         '''
-        if not arch:
-            raise RuntimeError("Must specify arch")
+        if arch == 'all':
+            arch = None
 
-        # debs with arch 'all' have to be categorized as a specific arch
-        # so use requested arch if not 'all', or system arch
-        fallback_arch = arch
-        if fallback_arch == 'all':
-            fallback_arch = host_architecture()
-
-        if self._have_all_binaries:
-            # Great!
-            pass
-        elif self.status in ["Pending", "Published"]:
+        if self.status in ["Pending", "Published"]:
             # Published, great!  Directly query the list of binaries
             binaries = map(BinaryPackagePublishingHistory,
                            self._lpobject.getPublishedBinaries())
             for b in binaries:
                 a = b.arch
-                if a == 'all':
-                    a = fallback_arch
                 if a not in self._binaries:
                     self._binaries[a] = {}
                 self._binaries[a][b.binary_package_name] = b
-            self._have_all_binaries = True
         else:
             # we have to go the long way :(
             Logger.info("Please wait, this may take some time...")
@@ -840,10 +829,12 @@ class SourcePackagePublishingHistory(BaseWrapper):
                 (pkgname, _, e) = filename.rpartition('.')
                 # split into name, version, arch
                 (n, v, a) = pkgname.rsplit('_', 2)
+                # arch 'all' has separate bpph for each real arch,
+                # but all point to the same binary url
                 if a == 'all':
-                    a = fallback_arch
+                    a = arch or host_architecture()
                 # Only check the arch requested - saves time
-                if arch != 'all' and arch != a:
+                if arch and arch != a:
                     continue
                 # Only check the name requested - saves time
                 if name and not re.match(name, n):
@@ -868,16 +859,13 @@ class SourcePackagePublishingHistory(BaseWrapper):
                 if a not in self._binaries:
                     self._binaries[a] = {}
                 self._binaries[a][n] = bpph
-            if not name and not ext and arch == 'all':
-                # We must have got them all
-                self._have_all_binaries = True
 
-        bpphs = []
-        if arch == 'all':
-            for a in self._binaries.values():
-                bpphs += a.values()
+        if not arch:
+            bpphs = [b for a in self._binaries.values() for b in a.values()]
         elif arch in self._binaries:
-            bpphs = self._binaries[arch].copy().values()
+            bpphs = list(self._binaries[arch].values())
+        else:
+            return []
 
         if name:
             bpphs = [b for b in bpphs if re.match(name, b.binary_package_name)]
